package com.ningdatech.pmapi.scheduler.task;

import cn.hutool.core.date.StopWatch;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.pmapi.common.constant.BizConst;
import com.ningdatech.pmapi.gov.manage.GovProjectCollectionManage;
import com.ningdatech.pmapi.gov.model.vo.GovBizProjectDetailVO;
import com.ningdatech.pmapi.gov.model.vo.GovBizProjectListVO;
import com.ningdatech.pmapi.projectlib.model.req.ProjectListReq;
import com.ningdatech.pmapi.scheduler.utils.DateUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * @author ZPF
 * @since 2023/08/31 18:16
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class SynProjectCollectionTask {

    @Value("${hostname}")
    public String HOST;

    @Value("${spring.profiles.active}")
    public String active;

    @Autowired
    private GovProjectCollectionManage collectionManage;

    private final Integer PAGE_NUMBER = 1;
    private final Integer TASK_HOUR = 1;

    private final Integer TASK_HOUR_TODAY = 18;

    /**
     * 前置机每天晚上10点自动清空，第二天早上6点获取数据
     * 定时同步前置机数据 每天1点开始执行一次
     *
     */
    @Scheduled(cron = "0 0 1 * * ?")
    public void doTask() throws UnknownHostException {
        if (!HOST.equals(InetAddress.getLocalHost().getHostName())) {
            log.info("定时器没开启或者host不对! {}:{}",
                    HOST,InetAddress.getLocalHost().getHostName());
        }
        if(BizConst.PRE.equals(active)){
            log.info("预发环境不用同步!");
            return;
        }
        log.info("数据同步任务开始");
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();

        //取昨天1点 到今天1点的 项目归集
        ProjectListReq req = new ProjectListReq();
        req.setCreateOnMax(DateUtil.convertDateToLocal(DateUtil.getTodayTime(TASK_HOUR_TODAY)));
        req.setCreateOnMin(DateUtil.convertDateToLocal(DateUtil.getTodayTime(TASK_HOUR)).plusDays(-1));
        Long count = collectionManage.count(req);
        if(count == 0L){
            log.info("本次同步无数据");
            return;
        }
        req.setPageNumber(PAGE_NUMBER);
        req.setPageSize(count.intValue());
        PageVo<GovBizProjectListVO> projectCollectionPage = collectionManage.list(req);

        log.info("projectCollections size :{}",projectCollectionPage.getTotal());

        for(GovBizProjectListVO vo : projectCollectionPage.getRecords()){
            GovBizProjectDetailVO projectDetail = collectionManage.detail(vo.getBaseProjId());
            collectionManage.pushProjectVo(projectDetail);
        }
        stopWatch.stop();
        log.info("数据同步任务结束====={}s",stopWatch.getTotalTimeSeconds());
    }
}
