package com.ningdatech.syndataapi.open.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.syndataapi.common.constant.BizConst;
import com.ningdatech.syndataapi.scheduler.model.dto.ProjectBaseInfoDTO;
import com.ningdatech.syndataapi.scheduler.model.dto.ProjectSaveDTO;
import com.ningdatech.syndataapi.scheduler.model.entity.*;
import com.ningdatech.syndataapi.scheduler.service.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

/**
 * @Classname ProjectReceiveManage
 * @Description
 * @Date 2023/8/25 9:11
 * @Author PoffyZhang
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class ProjectReceiveManage {

    private final IProjectBaseInfoService baseInfoService;

    private final IProjectApplyService applyService;

    private final IProjectApproveService approveService;

    private final IProjectCimplementService cimplementService;

    private final IProjectMimplementService mimplementService;

    private final IProjectProcureService procureService;

    /**
     * 接收保存 项目
     * @param dto
     * @return
     */
    public String save(ProjectSaveDTO dto) {

        //1.保存项目基本信息
        ProjectBaseInfo baseInfo = dto.getBaseinfo();
        String baseProjId = baseInfo.getBaseProjId();
        VUtils.isTrue(StringUtils.isBlank(baseProjId))
                .throwMessage("项目编号不能为空!");

        ProjectBaseInfo oldBaseInfo = baseInfoService.getOne(Wrappers.lambdaQuery(ProjectBaseInfo.class)
                .eq(ProjectBaseInfo::getBaseProjId, baseProjId)
                .last(BizConst.LIMIT_1));
        ProjectBaseInfo saveBase = BeanUtil.copyProperties(baseInfo,ProjectBaseInfo.class);
        if(Objects.isNull(oldBaseInfo)){
            saveBase.setOp(BizConst.OP_INSERT);
            saveBase.setTongTime(LocalDateTime.now());
        }else {
            saveBase.setOp(BizConst.OP_UPDATE);
        }
        saveBase.setBizTime(LocalDateTime.now());
        baseInfoService.saveOrUpdate(saveBase);

        // 2.保存 申报信息
        ProjectApply apply = dto.getApply();

        ProjectApply oldApply = applyService.getOne(Wrappers.lambdaQuery(ProjectApply.class)
                .eq(ProjectApply::getBaseProjId, baseProjId)
                .last(BizConst.LIMIT_1));

        ProjectApply saveApply = BeanUtil.copyProperties(apply,ProjectApply.class);
        saveApply.setBaseProjId(baseProjId);
        if(Objects.isNull(oldApply)){
            saveApply.setOp(BizConst.OP_INSERT);
            saveApply.setTongTime(LocalDateTime.now());
        }else{
            saveApply.setOp(BizConst.OP_UPDATE);
        }
        saveApply.setBizTime(LocalDateTime.now());
        applyService.saveOrUpdate(saveApply);

        // 3.保存 审批信息
        ProjectApprove approve = dto.getApprove();

        ProjectApprove oldApprove = approveService.getOne(Wrappers.lambdaQuery(ProjectApprove.class)
                .eq(ProjectApprove::getBaseProjId, baseProjId)
                .last(BizConst.LIMIT_1));

        ProjectApprove saveApprove = BeanUtil.copyProperties(approve,ProjectApprove.class);
        saveApprove.setBaseProjId(baseProjId);
        if(Objects.isNull(oldApprove)){
            saveApprove.setOp(BizConst.OP_INSERT);
            saveApprove.setTongTime(LocalDateTime.now());
        }else{
            saveApprove.setOp(BizConst.OP_UPDATE);
        }
        saveApprove.setBizTime(LocalDateTime.now());
        approveService.saveOrUpdate(saveApprove);


        // 4.保存 建设项目 实施信息
        ProjectCimplement cimplement = dto.getCimplement();
        if(Objects.nonNull(cimplement)){
            ProjectCimplement oldCimplement = cimplementService.getOne(Wrappers.lambdaQuery(ProjectCimplement.class)
                    .eq(ProjectCimplement::getBaseProjId, baseProjId)
                    .last(BizConst.LIMIT_1));

            ProjectCimplement saveCimplement = BeanUtil.copyProperties(cimplement,ProjectCimplement.class);
            saveCimplement.setBaseProjId(baseProjId);
            if(Objects.isNull(oldCimplement)){
                saveCimplement.setOp(BizConst.OP_INSERT);
                saveCimplement.setTongTime(LocalDateTime.now());
            }else{
                saveCimplement.setOp(BizConst.OP_UPDATE);
            }
            saveCimplement.setBizTime(LocalDateTime.now());
            cimplementService.saveOrUpdate(saveCimplement);
        }

        // 5.保存 运维项目 实施信息
        ProjectMimplement mimplement = dto.getMimplement();
        if(Objects.nonNull(mimplement)){
            ProjectMimplement oldMimplement = mimplementService.getOne(Wrappers.lambdaQuery(ProjectMimplement.class)
                    .eq(ProjectMimplement::getBaseProjId, baseProjId)
                    .last(BizConst.LIMIT_1));

            ProjectMimplement saveMimplement = BeanUtil.copyProperties(mimplement,ProjectMimplement.class);
            saveMimplement.setBaseProjId(baseProjId);
            if(Objects.isNull(oldMimplement)){
                saveMimplement.setOp(BizConst.OP_INSERT);
                saveMimplement.setTongTime(LocalDateTime.now());
            }else{
                saveMimplement.setOp(BizConst.OP_UPDATE);
            }
            saveMimplement.setBizTime(LocalDateTime.now());
            mimplementService.saveOrUpdate(saveMimplement);
        }

        // 6.保存 采购信息
        List<ProjectProcure> procures = dto.getProcures();
        if(Objects.nonNull(procures)){
//            procureService.remove(Wrappers.lambdaQuery(ProjectProcure.class)
//                    .eq(ProjectProcure::getBaseProjId,baseProjId));
            List<ProjectProcure> olds = procureService.list(Wrappers.lambdaQuery(ProjectProcure.class)
                    .eq(ProjectProcure::getBaseProjId, baseProjId));
            for(ProjectProcure old : olds){
                //删除标记
                old.setOp(BizConst.OP_DELETE);
                procureService.updateById(old);
            }
            for(ProjectProcure procure : procures){
                //添加标记
                ProjectProcure saveProcure = BeanUtil.copyProperties(procure,ProjectProcure.class);
                saveProcure.setBaseProjId(baseProjId);
                saveProcure.setTongID(null);
                saveProcure.setTongTime(LocalDateTime.now());
                saveProcure.setBizTime(LocalDateTime.now());
                saveProcure.setOp(BizConst.OP_INSERT);
                procureService.save(saveProcure);
            }
        }


        return BizConst.SAVE_SUCCESS;
    }

    /**
     * 置为无效
     * @param dto
     * @return
     */
    public String notEffective(ProjectBaseInfoDTO dto) {
        ProjectBaseInfo oldBaseInfo = baseInfoService.getOne(Wrappers.lambdaQuery(ProjectBaseInfo.class)
                .eq(ProjectBaseInfo::getBaseProjId, dto.getBaseProjId())
                .last(BizConst.LIMIT_1));

        ProjectBaseInfo saveBaseinfo = BeanUtil.copyProperties(dto,ProjectBaseInfo.class);
        saveBaseinfo.setIsEffective(BizConst.NOT_EFFECTIVE);
        if(Objects.isNull(oldBaseInfo)){
            saveBaseinfo.setTongTime(LocalDateTime.now());
        }
        saveBaseinfo.setBizTime(LocalDateTime.now());
        saveBaseinfo.setOp(BizConst.OP_UPDATE);
        baseInfoService.saveOrUpdate(saveBaseinfo);

        return BizConst.OP_SUCCESS;
    }
}
